#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <math.h>

#include "Graph.h"
#include "InputSet.h"

using namespace std;

CGraph::CGraph(int nV) {
  mNVertices = nV;
}

CGraph::CGraph(InputPurifications &instance) {
  
  mNVertices = instance.numberOfProteins();

//   mP = new double*[mNVertices];
//   for(int i=0; i < mNVertices; i++) {
//     mP[i] = new double[mNVertices];
//   }
  int nV = mNVertices;

  mLabels.resize(mNVertices);

  for(int i=0; i < mNVertices; i++) {
    mLabels[i] = instance.id2name[i];
  }

//   for(int i=0; i < mNVertices; i++) {
//     for(int j=i; j < mNVertices; j++) {
//       mP[i][j] = exp(instance.m_Posterior[j][i]);      
//     }
//   }
  mNodes.resize(mNVertices);
  for(int i=0; i < mNVertices; i++) 
    mNodes[i].v = i;

  int nEdges=0;
  for(int i=0; i < mNVertices; i++) {
    for(int j=i+1; j < mNVertices; j++) {
      int t,s;      
      t = instance.m_trials[j][i];
      s = instance.m_successes[j][i];

      if (t > 0) {
	mNodes[i].mAdjacency.push_back(Neighbor(j,t,s));
	mNodes[j].mAdjacency.push_back(Neighbor(i,t,s));
	nEdges++;
      }
    }
  }

  cout << "Number of edges = " << nEdges << endl;
}

CGraph::~CGraph() {
}

CSpokeGraph::CSpokeGraph(InputPurifications &instance, double tau) {
  
  mNVertices = instance.numberOfProteins();

  int nV = mNVertices;

  mLabels.resize(mNVertices);

  for(int i=0; i < mNVertices; i++) {
    mLabels[i] = instance.id2name[i];
  }

//   for(int i=0; i < mNVertices; i++) {
//     for(int j=i; j < mNVertices; j++) {
//       mP[i][j] = exp(instance.m_Posterior[j][i]);      
//     }
//   }
  mNodes.resize(mNVertices);
  for(int i=0; i < mNVertices; i++) {
    mNodes[i].v = i;
    mNodes[i].frequency = instance.Frequency[i];
  }

  int nEdges=0;
  for(int k=0; k < instance.Baits.size(); k++) {
    int i = instance.Baits[k];
    for(int j=0; j < mNVertices; j++) {   
      if (i != j && instance.purs[k].get_bit(j)) {
	double prob;
	if (i < j) 
	  prob = exp(instance.m_Posterior[j][i]);	
	else
	  prob = exp(instance.m_Posterior[i][j]);
	
	mNodes[i].mAdjacency.push_back(Neighbor(j,prob));
	mNodes[j].mAdjacency.push_back(Neighbor(i,prob));
	nEdges++;
      }
    }
  }

  cout << "Number of edges = " << nEdges << endl;
}

CRandomObs::CRandomObs(InputPurifications &instance, char *clusterfile) :
  CGraph(instance) {

  FILE *filep;

  filep = fopen(clusterfile, "r");
  
  char name[512];
  float lambda;
  fscanf(filep, "%s %f", name, &lambda); // header
  
  float rate;
  fscanf(filep, "%s %f", name, &rate);
  double fn = (double) rate;
  fscanf(filep, "%s %f", name, &rate);
  double fp = (double) rate;
  
  fscanf(filep, "%s %f", name, &rate); // Constant C
  
  int cluster;
  while( fscanf(filep, "%s %d", name, &cluster) != EOF) {
    name2cluster.insert( make_pair( string(name), cluster) ).first->second;    
    // cout << string(name) << "\t" << cluster << endl;
  }
  fclose(filep);

  mClasses.resize(mNVertices);
  for(int i=0; i < mNVertices; i++) {
    mClasses[i] = name2cluster[mLabels[i]];
  }

  // First, generate draw success parameters according to the rate
  for(int i=0; i < mNVertices; i++) {
    for(int j=0; j < mNodes[i].mAdjacency.size(); j++) {
      
      int t,s;                 
      int v = mNodes[i].mAdjacency[j].v;

      int ci = name2cluster[mLabels[i]];
      int cj = name2cluster[mLabels[v]];
      
      t = mNodes[i].mAdjacency[j].t; 
      s = 0;
      if (ci == cj) {      
	for(int k=0; k < t; k++) {		
	  double r = (double)rand()/((double)RAND_MAX+1.0);
	  if ( r > fn )
	    ++s;
	}
      } else {
	for(int k=0; k < t; k++) {		
	  double r = (double)rand()/((double)RAND_MAX+1.0);
	  if ( r < fp )
	    ++s;
	}
      }

      mNodes[i].mAdjacency[j].s = s;
    }
  }

}

CRandomCluster::CRandomCluster(int nV, int nK) : CGraph(nV) {
  
  mLabels.resize(mNVertices);
  for(int i=0; i < mNVertices; i++) {
    char buf[512];
    sprintf(buf, "%d", i+1);
    mLabels[i] = string(buf);
  }

  mClasses.resize(mNVertices);

  for(int i=0; i < mNVertices; i++) {
    mClasses[i] = (int)( (double)rand()*(double)nK/((double)RAND_MAX+1.0) );
    cout << i << " " << mClasses[i] << endl;
  }

  mNodes.resize(mNVertices);

  for(int i=0; i < mNVertices; i++) {
    for(int j=i+1; j < mNVertices; j++) {
      int t,s;

      if (mClasses[i] == mClasses[j]) {
	t = 10;
	s = 0;
	for(int i=0; i < 10; i++) {	
	  double r = (double)rand()/((double)RAND_MAX+1.0);
	  if ( r > 0.2 )
	    ++s;
	}
      } else {
	t = 5;
	s = 0;
	for(int i=0; i < 5; i++) {
	  double r = (double)rand()/((double)RAND_MAX+1.0);
	  if ( r < 0.001 )
	    ++s;
	}
      }
      
      mNodes[i].mAdjacency.push_back(Neighbor(j,t,s));
      mNodes[j].mAdjacency.push_back(Neighbor(i,t,s));
    }
  }
}

CFixSizes::CFixSizes(char *sizefile) {
  
  FILE *fp = fopen(sizefile, "r");

  char dummy[512];
  int s;
  vector<int> vSizes;

  mNVertices = 0;  
  while( fscanf(fp, "%s %d", dummy, &s) != EOF) {
    if (s > 1) {
      vSizes.push_back(s);
      mNVertices += s;
    }
  }
  cout << "Number of Vertices " << mNVertices << endl;
  cout << "Number of clusters " << vSizes.size() << endl;
  fclose(fp);
 
 
  mClasses.resize(mNVertices);

  int offset=0;
  for(int k=0; k < vSizes.size(); k++) {    
    for(int i=0; i < vSizes[k]; i++) {
      mClasses[offset + i] = k;
    }
    offset += vSizes[k];
  }

  mNodes.resize(mNVertices);

  for(int i=0; i < mNVertices; i++) {
    for(int j=i+1; j < mNVertices; j++) {
      int t,s;

      if (mClasses[i] == mClasses[j]) {
	t = 10;
	s = 0;
	for(int i=0; i < 10; i++) {	
	  double r = (double)rand()/((double)RAND_MAX+1.0);
	  if ( r > 0.2 )
	    ++s;
	}
      } else {
	t = 5;
	s = 0;
	for(int i=0; i < 5; i++) {
	  double r = (double)rand()/((double)RAND_MAX+1.0);
	  if ( r < 0.001 )
	    ++s;
	}
      }
      
      mNodes[i].mAdjacency.push_back(Neighbor(j,t,s));
      mNodes[j].mAdjacency.push_back(Neighbor(i,t,s));
    }
  }
}
  
