#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <math.h>
#include <map>
#include <string>

#include "InputSet.h"

using namespace std;

const int MAX_TRIALS = 10;

void Purification::set_bit(int j) {
  proteins[j] = true;
}

int Purification::get_bit(int j) {
  return proteins[j];
}

Purification::Purification(int nproteins) : proteins(nproteins, false) {
  
}

InputPurifications::~InputPurifications() {
  /******
  for(int i=0; i < name2id.size(); i++)
    delete [] m_Posterior[i];
  delete [] m_Posterior;

  delete [] m_proteinRate;
  *******/

  for(int i=0; i < name2id.size(); i++) { 
    delete [] m_trials[i];
    delete [] m_successes[i];
  }
  delete [] m_trials;
  delete [] m_successes;

}


void InputPurifications::readproteins(char *set_file) {
  char buf[10000];
	
  FILE *fp = fopen(set_file, "r");
	
  // this is the data structure that stores the input data
	
  while( fgets(buf, sizeof(buf)-1, fp ) != NULL ) {
    // cout << pur_id << " : " << buf << endl;

    // remove white spaces
    int len = strlen(buf);
    if (buf[len-1] == '\n') buf[len-1] = '\0';

    char const *token = strtok(buf, ",");

    int vid=name2id.insert( make_pair( string(token), name2id.size() ) ).first->second;
    Baits.push_back(vid);
    
    while( (token = strtok(NULL,",")) != NULL ) {
      vid=name2id.insert( make_pair( string(token), name2id.size() ) ).first->second;
    }
  }
	
  fclose(fp);

  numSets = Baits.size();
  
  cout << "Number of proteins = " << name2id.size() << endl;

  int nproteins = name2id.size();

  Frequency.resize(nproteins);
  for(int i=0; i < nproteins; i++) {
    Frequency[i] = 0;
  }

  fp = fopen(set_file, "r");
	
  // this is the data structure that stores the input data
  
  while( fgets(buf, sizeof(buf)-1, fp ) != NULL ) {
    // cout << pur_id << " : " << buf << endl;
    int len = strlen(buf);
    if (buf[len-1] == '\n') buf[len-1] = '\0';

    purs.push_back(Purification(nproteins));
    int k = purs.size()-1;
    char const *token = strtok(buf, ",");
    
    int vid=name2id.find(string(token))->second;
    purs[k].set_bit(vid);
    ++Frequency[vid];

    while( (token = strtok(NULL,",")) != NULL ) {
      vid=name2id.find(string(token))->second;
      purs[k].set_bit(vid);
      ++Frequency[vid];
    }
  }
  fclose(fp);

  id2name.resize(name2id.size());
  map<string,int,less<string> >::iterator pos;
  for(pos = name2id.begin(); pos != name2id.end(); ++pos) {
    // cout << pos->first << "\t" << pos->second << endl;
    id2name[pos->second] = pos->first;
  }
}	

void InputPurifications::CountMatrixModel() {

  int i,j;
  int nBaits = Baits.size();
  int nproteins = name2id.size();
  
  m_trials = new int*[nproteins];
  for(i=0; i < nproteins; i++) {
    m_trials[i] = new int[nproteins];
  }
	
  m_successes = new int*[nproteins];
  for(i=0; i < nproteins; i++) {
    m_successes[i] = new int[nproteins];
  }
	
  for(i = 0; i < nproteins; i++) {
    for(j=0; j < nproteins; j++) {
      m_trials[i][j] = 0;
      m_successes[i][j] = 0;
    }
  }

  for(i = 0; i < purs.size(); i++) {
    for(int k = 0; k < nproteins; k++) {      
      int a = purs[i].get_bit(k);
      for(int z = 0; z < k; z++) {
	int b = purs[i].get_bit(z);
	
	m_trials[k][z] += ((a == 1) || (b == 1));   
	m_successes[k][z] += ((a == 1) && (b == 1));   
      } 
    }
  }
 
  /*********
  for(i = 0; i < nproteins; i++) 
    for(j=0; j < i; j++) {
      cout << id2name[i] << "," << id2name[j];
      cout << " " << m_trials[i][j];
      cout << " " << m_successes[i][j] << endl;
    }
  *********/
}

void InputPurifications::generateCounts() {
	
  int i,j;
  int nBaits = Baits.size();
  int nproteins = name2id.size();

  m_trials = new int*[nproteins];
  for(i=0; i < nproteins; i++) {
    m_trials[i] = new int[nproteins];
  }
	
  m_successes = new int*[nproteins];
  for(i=0; i < nproteins; i++) {
    m_successes[i] = new int[nproteins];
  }
	
  for(i = 0; i < nproteins; i++) {
    for(j=0; j < nproteins; j++) {
      m_trials[i][j] = 0;
      m_successes[i][j] = 0;
    }
  }
	
  for(i = 0; i < nBaits; i++) {
    int bait = Baits[i];
    for(j = 0; j <= bait; j++) {
      ++m_trials[bait][j];
    }
    for(j = bait+1; j < nproteins; j++) {
      ++m_trials[j][bait];
    }
  }

  for(i = 0; i < nBaits; i++) {
    int bait = Baits[i];
    for(j = 0; j < nproteins; j++) {
      if (bait == j) {
	++m_successes[j][j];
      } else {	
	if ( purs[i].get_bit(j) ) {
	  if (bait < j) {
	    ++m_successes[j][bait];
	  } else {
	    ++m_successes[bait][j];
	  }
	}
      }
    }
  }
}

double InputPurifications::trueInteraction(int t, int s) {
  return bicoeff(t,s)*pow(m_Nu,t-s)*pow(1.0 - m_Nu, s);
}

double InputPurifications::falseInteraction(int t, int s) {
  return bicoeff(t,s)*pow(1.0 - m_Phi, t-s) * pow(m_Phi, s);
} 

void InputPurifications::interactionProbability() {
  double posterior[MAX_TRIALS][MAX_TRIALS];
  int t,s;
	
  cout << "Posterior probability\n";
  for(t = 0; t < MAX_TRIALS; t++) {
    cout << t;
    for(s = 0; s <= t; s++) {
      posterior[t][s] = log(trueInteraction(t,s)*m_Rho);
      posterior[t][s] -= log(trueInteraction(t,s)*m_Rho + falseInteraction(t,s)*(1.0-m_Rho));
      cout << " " << exp(posterior[t][s]);
    }
    cout << endl;
  }
	
  int NPROTEINS = name2id.size();
  int i,j;
  m_Posterior = new double*[NPROTEINS];
  for(i = 0; i < NPROTEINS; i++) {
    m_Posterior[i] = new double[NPROTEINS];
  }
	
  for(i = 0; i < NPROTEINS; i++) {
    for(j = 0; j <= i; j++) {
      t = m_trials[i][j];
      s = m_successes[i][j];
      m_Posterior[i][j] = posterior[t][s];				
    }
  }

  m_proteinRate = new double[NPROTEINS];
  for(i = 0; i < NPROTEINS; i++) {
    m_proteinRate[i] = 0.0;
    for(j = 0; j < Baits.size(); j++) {      
      if (Baits[j] > i)
	m_proteinRate[i] += exp(m_Posterior[Baits[j]][i]);
      else
	m_proteinRate[i] += exp(m_Posterior[i][Baits[j]]);
    }
    
    m_proteinRate[i] /= float(Baits.size());
    m_proteinRate[i] = log(m_proteinRate[i]);
  }
}

void InputPurifications::computeNoiseModel(double nu, double phi, double rho) {	
  m_Nu = nu;
  m_Phi = phi;
  m_Rho = rho;
	
  generateCounts();
	
  // Compute a table to store the posterior
  interactionProbability();
    
  int i,j;
  for(i = 0; i < name2id.size(); i++) {
    for(j = 0; j <= i; j++) {
      //cout << m_Posterior[i][j] << " ";				
    }
    //cout << endl;
  }
  
}

int InputPurifications::numberOfProteins() {
  return name2id.size();
}

#include "Graph.h"

void InputPurifications::Output(char *outfile) {

  FILE *fp = fopen(outfile, "w");

  int nproteins = name2id.size();

  for(int i = 0; i < nproteins; i++) 
    for(int j=0; j < i; j++) {

      double t = m_trials[i][j];
      double s = m_successes[i][j];
      double prob = exp(m_Posterior[i][j]);

      if (t > 0.0) {
	fprintf(fp, "%s %s %1.5f\n", id2name[i].c_str(), id2name[j].c_str(), prob);
      }
    }  
  

  fclose(fp);
}

  
