
#ifndef _INPUTSET_H
#define _INPUTSET_H

#include <stdlib.h>
#include <stdio.h>
#include <vector>
#include <map>
#include <string>

using namespace std;

const int NSETS = 2000;

const unsigned int NBITSPERINT = 8*sizeof(int);

class Purification {
public:
  Purification();
  Purification(int nproteins);

  void set_bit(int j);
  int get_bit(int j);
  
 private:
  vector<bool> proteins;
};

extern double bicoeff(int n,int k);

class InputPurifications {

 private:
	double m_beta; // false negative rate
	vector<double> m_fpRate;
	double m_Nu;
	double m_Phi;
	double m_Rho;
	
	double *m_Prior;

	
	map<string, int, less<string> > name2id;

	double trueInteraction(int t, int s);
	double falseInteraction(int t, int s);
	void interactionProbability();
	
 public:
	~InputPurifications();
	void readsets(char *set_file);
	void readproteins(char *set_file);
	void generateCounts();
	void computeNoiseModel(double nu, double phi, double rho);
	void CountMatrixModel();
	int numberOfProteins();
	void Output(char *outfile);

	vector<int> Baits;
 	vector<Purification> purs;
	int **m_trials;
	int **m_successes;
	double **m_Posterior;
	double *m_proteinRate;
	int numSets;
	vector<string> id2name;
	vector<int> Frequency;

};

#endif
