#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <time.h>

#include "Graph.h"
#include "RandomField.h"
#include "InputSet.h"

using namespace std;

void writeClusters(char *filename, CGraph &G, CAnnealing &CRF) {
  int i;

  FILE *fp = fopen(filename, "w");

  fprintf(fp, "Lambda\t%.5f\n", CRF.mLambda);
  fprintf(fp, "FN\t%.5f\n", CRF.mFalseNegative);
  fprintf(fp, "FP\t%.5f\n", CRF.mFalsePositive);
  fprintf(fp, "Constant\t%.5f\n", CRF.mPsi);

  for(int i=0; i < G.mNVertices; i++) {    
    fprintf(fp,"%s\t%d\n", G.mLabels[i].c_str(), CRF.mGroups[i]);
  }
  fclose(fp);
}

void usage() {
  fprintf(stderr, "Usage: estimate [-m] Purification  K psi Output-name \n ");
  fprintf(stderr, "Option: \n");
  fprintf(stderr, "\t m: use the matrix model (default: spoke model) \n ");
  fprintf(stderr, "Arguments: \n");
  fprintf(stderr, "\t Purification: input purification file (comma separated file) \n ");
  fprintf(stderr, "\t K: number of clusters \n ");
  fprintf(stderr, "\t psi: log ratio of the error rate. We recommend values between 2.0-15.0 \n ");
  fprintf(stderr, "\t Output: the name of the output file for cluster assignment of each protein. \n ");  
}

int main(int argc, char *argv[]) {
  int c;
  int matrix_flag = 0;

  if (argc < 5) {
    usage();
    exit(-1);
  }

  while((c = getopt(argc, argv, "m")) != -1) {
    switch(c) {
    case 'm':
      matrix_flag = 1;
      break;
    case '?': {
      fprintf(stderr, "Unknown option \n ");
      usage();
      return 1;      
    }
    default:
      usage();
      exit(-1);
    }
  }

  int index;
  for(index=optind; index < argc; index++)
    printf( "Non-option argument %s\n", argv[index]);

  srand(time(0));

  InputPurifications purs;
  purs.readproteins(argv[optind]); 

  if (matrix_flag) 
    purs.CountMatrixModel();
  else
    purs.computeNoiseModel(0.346, 0.00107, 0.00188);  
  
  CGraph G(purs);
  
  int K = atoi(argv[optind + 1]);
  double beta = (double)atof(argv[optind + 2]);
  CAnnealing CRF(G.mNVertices,K, beta);
  
  CRF.assignment(G, 100);

  //char outname[512];
  //sprintf(outname, "Results06/clusters.%d.Beta2.out", K);
  writeClusters(argv[optind + 3], G, CRF);
}
